==> sumika-0.10.tar.gz: sumika-0.10/common/cannadic.c <==
/*
 *  cannadic.c
 *  Copyright(C) 2003- Masahito Omote <omote@utyuuzin.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
 
#include "cannadic.h"
#include "dixchange.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

void cannadic_parse_line(unsigned char *buf, word **head) {
    /*
     * cannadic format:
     *
     * phon cclass desc (desc desc ... ) (cclass desc (desc ...) ....)
     *
     * phon  : must
     * cclass: at least 1 entry
     * desc  : at least 1 entry
     *
     * cclass and desc must be associated with each other.
     *
     */
    unsigned char *pos = NULL, *phonetic = NULL, *literal = NULL;
    unsigned char *cur = NULL, *cur1 = NULL;
    unsigned char *tmp1 = NULL, *tmp2 = NULL;
    int freq = 1;

    if(buf[strlen(buf) - 1] == '\n') buf[strlen(buf) - 1] = '\0';

    cur = strchr(buf, ' ');
    if(cur != NULL) {
	*cur = '\0';
	cur++;
    }
    phonetic = strdup(buf);

    while(1) {
	cur1 = strchr(cur, ' ');
	if(cur1 != NULL) {
	    *cur1 = '\0';
	}

	if(cur[0] == '#') {
	    freq = 1;
	    tmp1 = strdup(cur);
	    tmp2 = strchr(tmp1, '*');
	    if(tmp2 != NULL) {
		*tmp2 = '\0';
		tmp2++;
		freq = atoi(tmp2);
	    }
	    pos = tmp1;
	} else {
	    literal = cur;
	    word_append(head, WORD_TYPE_CANNA,
			phonetic, literal, pos,
			    0, 0, NULL);
	}
	if(cur1 == NULL) break;
	
	cur = cur1;
	cur++;
    }
}

GList *cannadic_parse_line_glist(unsigned char *buf, GList *list) {
    /*
     * cannadic format:
     *
     * phon cclass desc (desc desc ... ) (cclass desc (desc ...) ....)
     *
     * phon  : must
     * cclass: at least 1 entry
     * desc  : at least 1 entry
     *
     * cclass and desc must be associated with each other.
     *
     */
    unsigned char *pos = NULL, *phonetic = NULL, *literal = NULL;
    unsigned char *cur = NULL, *cur1 = NULL;
    unsigned char *tmp1 = NULL, *tmp2 = NULL;
    gchar *utf8_literal, *utf8_phonetic;
    gchar score[1024], cclass[1024];
    gint rbytes, wbytes;
    int freq = 1;

    if(buf[strlen(buf) - 1] == '\n') buf[strlen(buf) - 1] = '\0';

    cur = strchr(buf, ' ');
    if(cur != NULL) {
	*cur = '\0';
	cur++;
    }
    phonetic = strdup(buf);

    while(1) {
	cur1 = strchr(cur, ' ');
	if(cur1 != NULL) {
	    *cur1 = '\0';
	}

	if(cur[0] == '#') {
	    freq = 1;
	    tmp1 = strdup(cur);
	    tmp2 = strchr(tmp1, '*');
	    if(tmp2 != NULL) {
		*tmp2 = '\0';
		tmp2++;
		freq = atoi(tmp2);
	    }
	    pos = tmp1;
	} else {
	    literal = cur;
	    utf8_phonetic = g_convert(phonetic, -1, "UTF-8", "EUC-JP", &rbytes, &wbytes, NULL);
	    utf8_literal = g_convert(literal, -1, "UTF-8", "EUC-JP", &rbytes, &wbytes, NULL);
	    snprintf(score, sizeof(score), "%d", freq);
	    cclass[0] = '\0';
	    list = dixchange_append_word(list, literal, phonetic, pos, cclass, score);
	    g_free(utf8_phonetic);
	    g_free(utf8_literal);
	    g_free(tmp1);
	}
	if(cur1 == NULL) break;
	cur = cur1;
	cur++;
    }

    return list;
}

int cannadic_import(const char* filename, int imtype)
{
    FILE *fp;
    char buf[1024];
    int len;
    word *list = NULL, *pos = NULL;
	
    if(filename != NULL && filename[0] != '\0') {
	fp = fopen(filename, "r");
	if(!fp) {
	    return -1;
	}
	while(fgets(buf, sizeof(buf), fp)) {
	    len = strlen(buf);
	    buf[len-1] = '\0';
	    if(buf[0] != '#') {
		cannadic_parse_line(buf, &list);
	    }
	}
	for(pos = list; pos != NULL; pos = pos->next) {
	    switch (imtype) {
	    case 0: /* Anthy */
		add_anthy_priv_dic_with_flags(pos->phon,
					      pos->desc,
					      pos->cclass_code, 1);
		break;
	    case 1: /* Canna */
		snprintf(buf, sizeof(buf), "%s %s %s", pos->phon, pos->cclass_code, pos->desc);
		canna_define_dic("user", buf);
		break;
	    case 2: /* PRIME */
	    case 3: /* SKK */
	    default:
		break;
	    }
	}
	fclose(fp);
    }
}

char *find_pos_from_code(const char *code, int type) {
    /* need to be more smart */
    int i = 0, j = 0;
    char *pos = NULL;
    category_code *category[] = {
	substantive_code,
	verb_code,
	adjective_code,
	adverb_code,
	etc_code,
	NULL,
    };
    int num[] = { /* XXX */
	sizeof(substantive_code) / sizeof(substantive_code[0]),
	sizeof(verb_code) / sizeof(verb_code[0]),
	sizeof(adjective_code) / sizeof(adjective_code[0]),
	sizeof(adverb_code) / sizeof(adverb_code[0]),
	sizeof(etc_code) / sizeof(etc_code[0]),
	0,
    };

    do {
	for(j = 0; j < num[i]; j++) { /* XXX */
	    if(strcmp(code, (category[i])[j].code) == 0
	       && (category[i])[j].type & type == type)
		pos = strdup((category[i])[j].desc);
	}
	i++;
    } while(category[i] != NULL);

    return pos;
}

char *find_code_from_pos(const char *pos, int type) {
    /* need to be more smart */
    int i = 0, j = 0;
    char *code = NULL;
    category_code *category[] = {
	substantive_code,
	verb_code,
	adjective_code,
	adverb_code,
	etc_code,
	NULL,
    };
    int num[] = { /* XXX */
	sizeof(substantive_code) / sizeof(substantive_code[0]),
	sizeof(verb_code) / sizeof(verb_code[0]),
	sizeof(adjective_code) / sizeof(adjective_code[0]),
	sizeof(adverb_code) / sizeof(adverb_code[0]),
	sizeof(etc_code) / sizeof(etc_code[0]),
	0,
    };

    do {
	for(j = 0; j < num[i]; j++) { /* XXX */
	    if(strcmp(pos, (category[i])[j].desc) == 0
	       && (category[i])[j].type & type == type)
		code = strdup((category[i])[j].code);
	}
	i++;
    } while(category[i] != NULL);

    return code;
}

int cannadic_export(const char *filename, int type)
{
    GList *list, *pos;
    FILE *fp;

    switch(type) {
    case 0: /* Anthy */
	list = anthy_read_privatediclist();
	break;
    case 1: /* Canna */
	list = canna_read_privatediclist();
	break;
    default:
	break;
    }

    fp = fopen(filename, "w");
    if(fp == NULL)
	return -1;

    for(pos = g_list_first(list); pos != NULL; pos = g_list_next(pos))
    {
	char line[1024];
	char *euc_line;
	int score, rbytes, wbytes;
	dixchange_word *word;

	word = pos->data;
	if(word->score != 1)
	{
	    snprintf(line, sizeof(line), "%s %s*%s %s",
		     word->phonetic, word->pos, word->score, word->literal);
	} else {
	    snprintf(line, sizeof(line), "%s %s %s",
		     word->phonetic, word->pos, word->literal);
	}
	euc_line = g_convert(line, -1, "EUC-JP", "UTF-8", &rbytes, &wbytes, NULL);
	fprintf(fp, "%s\n", euc_line);
	free(euc_line);
    }
    fclose(fp);

    return 0;
}

==> sumika_0.11.orig.tar.gz: sumika-0.11/common/cannadic.c <==
/*
 *  cannadic.c
 *  Copyright(C) 2003- Masahito Omote <omote@utyuuzin.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
 
#include "cannadic.h"
#include "dixchange.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

void cannadic_parse_line(unsigned char *buf, word **head) {
    /*
     * cannadic format:
     *
     * phon cclass desc (desc desc ... ) (cclass desc (desc ...) ....)
     *
     * phon  : must
     * cclass: at least 1 entry
     * desc  : at least 1 entry
     *
     * cclass and desc must be associated with each other.
     *
     */
    unsigned char *pos = NULL, *phonetic = NULL, *literal = NULL;
    unsigned char *cur = NULL, *cur1 = NULL;
    unsigned char *tmp1 = NULL, *tmp2 = NULL;
    int freq = 1;

    if(buf[strlen(buf) - 1] == '\n') buf[strlen(buf) - 1] = '\0';

    cur = strchr(buf, ' ');
    if(cur != NULL) {
	*cur = '\0';
	cur++;
    }
    phonetic = strdup(buf);

    while(1) {
	cur1 = strchr(cur, ' ');
	if(cur1 != NULL) {
	    *cur1 = '\0';
	}

	if(cur[0] == '#') {
	    freq = 1;
	    tmp1 = strdup(cur);
	    tmp2 = strchr(tmp1, '*');
	    if(tmp2 != NULL) {
		*tmp2 = '\0';
		tmp2++;
		freq = atoi(tmp2);
	    }
	    pos = tmp1;
	} else {
	    literal = cur;
	    word_append(head, WORD_TYPE_CANNA,
			phonetic, literal, pos,
			    0, 0, NULL);
	}
	if(cur1 == NULL) break;
	
	cur = cur1;
	cur++;
    }
}

GList *cannadic_parse_line_glist(unsigned char *buf, GList *list) {
    /*
     * cannadic format:
     *
     * phon cclass desc (desc desc ... ) (cclass desc (desc ...) ....)
     *
     * phon  : must
     * cclass: at least 1 entry
     * desc  : at least 1 entry
     *
     * cclass and desc must be associated with each other.
     *
     */
    unsigned char *pos = NULL, *phonetic = NULL, *literal = NULL;
    unsigned char *cur = NULL, *cur1 = NULL;
    unsigned char *tmp1 = NULL, *tmp2 = NULL;
    gchar *utf8_literal, *utf8_phonetic;
    gchar score[1024], cclass[1024];
    gint rbytes, wbytes;
    int freq = 1;

    if(buf[strlen(buf) - 1] == '\n') buf[strlen(buf) - 1] = '\0';

    cur = strchr(buf, ' ');
    if(cur != NULL) {
	*cur = '\0';
	cur++;
    }
    phonetic = strdup(buf);

    while(1) {
	cur1 = strchr(cur, ' ');
	if(cur1 != NULL) {
	    *cur1 = '\0';
	}

	if(cur[0] == '#') {
	    freq = 1;
	    tmp1 = strdup(cur);
	    tmp2 = strchr(tmp1, '*');
	    if(tmp2 != NULL) {
		*tmp2 = '\0';
		tmp2++;
		freq = atoi(tmp2);
	    }
	    pos = tmp1;
	} else {
	    GError *err1, *err2;
	    literal = cur;
	    utf8_phonetic = g_convert(phonetic, -1, "UTF-8", "EUC-JP", &rbytes, &wbytes, &err1);
	    utf8_literal = g_convert(literal, -1, "UTF-8", "EUC-JP", &rbytes, &wbytes, &err2);
	    snprintf(score, sizeof(score), "%d", freq);
	    cclass[0] = '\0';

	    if(err1->code != G_CONVERT_ERROR_FAILED && err2->code != G_CONVERT_ERROR_FAILED)
		list = dixchange_append_word(list, literal, phonetic, pos, cclass, score);

	    if(utf8_phonetic != NULL)
		g_free(utf8_phonetic);
	    if(utf8_literal != NULL)
		g_free(utf8_literal);
	    if(tmp1 != NULL)
		g_free(tmp1);
	}
	if(cur1 == NULL) break;
	cur = cur1;
	cur++;
    }

    return list;
}

int cannadic_import(const char* filename, int imtype)
{
    FILE *fp;
    char buf[1024];
    int len;
    word *list = NULL, *pos = NULL;
	
    if(filename != NULL && filename[0] != '\0') {
	fp = fopen(filename, "r");
	if(!fp) {
	    return -1;
	}
	while(fgets(buf, sizeof(buf), fp)) {
	    len = strlen(buf);
	    buf[len-1] = '\0';
	    if(buf[0] != '#') {
		cannadic_parse_line(buf, &list);
	    }
	}
	for(pos = list; pos != NULL; pos = pos->next) {
	    switch (imtype) {
	    case 0: /* Anthy */
		add_anthy_priv_dic_with_flags(pos->phon,
					      pos->desc,
					      pos->cclass_code, 1);
		break;
	    case 1: /* Canna */
		snprintf(buf, sizeof(buf), "%s %s %s", pos->phon, pos->cclass_code, pos->desc);
		canna_define_dic("user", buf);
		break;
	    case 2: /* PRIME */
	    case 3: /* SKK */
	    default:
		break;
	    }
	}
	fclose(fp);
    }
}

char *find_pos_from_code(const char *code, int type) {
    /* need to be more smart */
    int i = 0, j = 0;
    char *pos = NULL;
    category_code *category[] = {
	substantive_code,
	verb_code,
	adjective_code,
	adverb_code,
	etc_code,
	NULL,
    };
    int num[] = { /* XXX */
	sizeof(substantive_code) / sizeof(substantive_code[0]),
	sizeof(verb_code) / sizeof(verb_code[0]),
	sizeof(adjective_code) / sizeof(adjective_code[0]),
	sizeof(adverb_code) / sizeof(adverb_code[0]),
	sizeof(etc_code) / sizeof(etc_code[0]),
	0,
    };

    do {
	for(j = 0; j < num[i]; j++) { /* XXX */
	    if(strcmp(code, (category[i])[j].code) == 0
	       && (category[i])[j].type & type == type)
		pos = strdup((category[i])[j].desc);
	}
	i++;
    } while(category[i] != NULL);

    return pos;
}

char *find_code_from_pos(const char *pos, int type) {
    /* need to be more smart */
    int i = 0, j = 0;
    char *code = NULL;
    category_code *category[] = {
	substantive_code,
	verb_code,
	adjective_code,
	adverb_code,
	etc_code,
	NULL,
    };
    int num[] = { /* XXX */
	sizeof(substantive_code) / sizeof(substantive_code[0]),
	sizeof(verb_code) / sizeof(verb_code[0]),
	sizeof(adjective_code) / sizeof(adjective_code[0]),
	sizeof(adverb_code) / sizeof(adverb_code[0]),
	sizeof(etc_code) / sizeof(etc_code[0]),
	0,
    };

    do {
	for(j = 0; j < num[i]; j++) { /* XXX */
	    if(strcmp(pos, (category[i])[j].desc) == 0
	       && (category[i])[j].type & type == type)
		code = strdup((category[i])[j].code);
	}
	i++;
    } while(category[i] != NULL);

    return code;
}

int cannadic_export(const char *filename, int type)
{
    GList *list, *pos;
    FILE *fp;

    switch(type) {
    case 0: /* Anthy */
	list = anthy_read_privatediclist();
	break;
    case 1: /* Canna */
	list = canna_read_privatediclist();
	break;
    default:
	break;
    }

    fp = fopen(filename, "w");
    if(fp == NULL)
	return -1;

    for(pos = g_list_first(list); pos != NULL; pos = g_list_next(pos))
    {
	char line[1024];
	char *euc_line;
	int score, rbytes, wbytes;
	dixchange_word *word;

	word = pos->data;
	if(word->score != 1)
	{
	    snprintf(line, sizeof(line), "%s %s*%s %s",
		     word->phonetic, word->pos, word->score, word->literal);
	} else {
	    snprintf(line, sizeof(line), "%s %s %s",
		     word->phonetic, word->pos, word->literal);
	}
	euc_line = g_convert(line, -1, "EUC-JP", "UTF-8", &rbytes, &wbytes, NULL);
	fprintf(fp, "%s\n", euc_line);
	free(euc_line);
    }
    fclose(fp);

    return 0;
}
